@@ -7,7 +7,7 @@ from django.db import models
 class Migration(SchemaMigration):
 
     def forwards(self, orm):
-        
+
         # Adding model 'ProductRecommendation'
         db.create_table('catalogue_productrecommendation', (
             ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
@@ -40,8 +40,10 @@ class Migration(SchemaMigration):
             ('depth', self.gf('django.db.models.fields.PositiveIntegerField')()),
             ('numchild', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
             ('name', self.gf('django.db.models.fields.CharField')(max_length=255, db_index=True)),
-            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=1024, db_index=True)),
-            ('full_name', self.gf('django.db.models.fields.CharField')(max_length=1024, db_index=True)),
+            ('slug',
+             self.gf('django.db.models.fields.SlugField')(max_length=255, db_index=True)),
+            ('full_name',
+             self.gf('django.db.models.fields.CharField')(max_length=255, db_index=True)),
         ))
         db.send_create_signal('catalogue', ['Category'])
 
@@ -197,7 +199,7 @@ class Migration(SchemaMigration):
 
 
     def backwards(self, orm):
-        
+
         # Removing unique constraint on 'ProductImage', fields ['product', 'display_order']
         db.delete_unique('catalogue_productimage', ['product_id', 'display_order'])
 
@@ -287,12 +289,14 @@ class Migration(SchemaMigration):
         'catalogue.category': {
             'Meta': {'ordering': "['name']", 'object_name': 'Category'},
             'depth': ('django.db.models.fields.PositiveIntegerField', [], {}),
-            'full_name': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'db_index': 'True'}),
+            'full_name': ('django.db.models.fields.CharField', [],
+                          {'max_length': '255', 'db_index': 'True'}),
             'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
             'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
             'numchild': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
             'path': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
-            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '1024', 'db_index': 'True'})
+            'slug': ('django.db.models.fields.SlugField', [], {'max_length':
+                                                               '255', 'db_index': 'True'})
         },
         'catalogue.contributor': {
             'Meta': {'object_name': 'Contributor'},
