@@ -2,30 +2,18 @@ from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.db import models
 from django.db.models import Sum, Count
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy as _
 
 from oscar.apps.catalogue.reviews.managers import ApprovedReviewsManager
+from oscar.core.compat import AUTH_USER_MODEL
+from oscar.core import validators
 
 
 class AbstractProductReview(models.Model):
     """
-    Superclass ProductReview.
+    A review of a product
 
-    Some key aspects have been implemented from the original spec.
-    * Each product can have reviews attached to it. Each review has a title, a
-      body and a score from 0-5.
-    * Signed in users can always submit reviews, anonymous users can only
-      submit reviews if a setting OSCAR_ALLOW_ANON_REVIEWS is set to true - it
-      should default to false.
-    * If anon users can submit reviews, then we require their name, email
-      address and an (optional) URL.
-    * By default, reviews must be approved before they are live.
-      However, if a setting OSCAR_MODERATE_REVIEWS is set to false,
-      then they don't need moderation.
-    * Each review should have a permalink, ie it has its own page.
-    * Each reviews can be voted up or down by other users
-    * Only signed in users can vote
-    * A user can only vote once on each product once
+    Reviews can belong to a user or be anonymous.
     """
 
     # Note we keep the review even if the product is deleted
@@ -37,12 +25,16 @@ class AbstractProductReview(models.Model):
     SCORE_CHOICES = tuple([(x, x) for x in range(0, 6)])
     score = models.SmallIntegerField(_("Score"), choices=SCORE_CHOICES)
 
-    title = models.CharField(max_length=255, verbose_name=_("Review title"))
+    title = models.CharField(
+        max_length=255, verbose_name=_("Review title"),
+        validators=[validators.non_whitespace])
     body = models.TextField(_("Body"))
 
-    # User information.  We include fields to handle anonymous users
+    # User information.
     user = models.ForeignKey(
-        'auth.User', related_name='reviews', null=True, blank=True)
+        AUTH_USER_MODEL, related_name='reviews', null=True, blank=True)
+
+    # Fields to be completed if user is anonymous
     name = models.CharField(_("Name"), max_length=255, null=True, blank=True)
     email = models.EmailField(_("Email"), null=True, blank=True)
     homepage = models.URLField(_("URL"), null=True, blank=True)
@@ -53,7 +45,9 @@ class AbstractProductReview(models.Model):
         (APPROVED, _("Approved")),
         (REJECTED, _("Rejected")),
     )
-    default_status = FOR_MODERATION if settings.OSCAR_MODERATE_REVIEWS else APPROVED
+    default_status = APPROVED
+    if settings.OSCAR_MODERATE_REVIEWS:
+        default_status = FOR_MODERATION
     status = models.SmallIntegerField(
         _("Status"), choices=STATUS_CHOICES, default=default_status)
 
@@ -73,8 +67,8 @@ class AbstractProductReview(models.Model):
         abstract = True
         ordering = ['-delta_votes']
         unique_together = (('product', 'user'),)
-        verbose_name = _('Product Review')
-        verbose_name_plural = _('Product Reviews')
+        verbose_name = _('Product review')
+        verbose_name_plural = _('Product reviews')
 
     @models.permalink
     def get_absolute_url(self):
@@ -86,31 +80,70 @@ class AbstractProductReview(models.Model):
     def __unicode__(self):
         return self.title
 
-    def save(self, *args, **kwargs):
+    def clean(self):
+        self.title = self.title.strip()
+        self.body = self.body.strip()
         if not self.user and not (self.name and self.email):
-            raise ValidationError(_("Anonymous review must have a name and an email"))
-        if not self.title:
-            raise ValidationError(_("Reviews must have a title"))
-        if self.score is None:
-            raise ValidationError(_("Reviews must have a score"))
+            raise ValidationError(
+                _("Anonymous reviews must include a name and an email"))
+
+    def vote_up(self, user):
+        self.votes.create(user=user, delta=AbstractVote.UP)
+
+    def vote_down(self, user):
+        self.votes.create(user=user, delta=AbstractVote.DOWN)
+
+    def save(self, *args, **kwargs):
         super(AbstractProductReview, self).save(*args, **kwargs)
         self.product.update_rating()
 
     def delete(self, *args, **kwargs):
         super(AbstractProductReview, self).delete(*args, **kwargs)
-        self.product.update_rating()
+        if self.product is not None:
+            self.product.update_rating()
 
+    # Properties
+
+    @property
+    def is_anonymous(self):
+        return self.user is None
+
+    @property
+    def pending_moderation(self):
+        return self.status == self.FOR_MODERATION
+
+    @property
+    def is_approved(self):
+        return self.status == self.APPROVED
+
+    @property
+    def is_rejected(self):
+        return self.status == self.REJECTED
+
+    @property
     def has_votes(self):
         return self.total_votes > 0
 
+    @property
     def num_up_votes(self):
         """Returns the total up votes"""
         return int((self.total_votes + self.delta_votes) / 2)
 
+    @property
     def num_down_votes(self):
         """Returns the total down votes"""
         return int((self.total_votes - self.delta_votes) / 2)
 
+    @property
+    def reviewer_name(self):
+        if self.user:
+            name = self.user.get_full_name()
+            return name if name else _('anonymous')
+        else:
+            return self.name
+
+    # Helpers
+
     def update_totals(self):
         """
         Update total and delta votes
@@ -121,22 +154,30 @@ class AbstractProductReview(models.Model):
         self.delta_votes = result['score'] or 0
         self.save()
 
-    def get_reviewer_name(self):
-        if self.user:
-            name = self.user.get_full_name()
-            return name if name else _('anonymous')
-        else:
-            return self.name
+    def can_user_vote(self, user):
+        """
+        Test whether the passed user is allowed to vote on this
+        review
+        """
+        if not user.is_authenticated():
+            return False, u"Only signed in users can vote"
+        vote = self.votes.model(review=self, user=user, delta=1)
+        try:
+            vote.full_clean()
+        except ValidationError, e:
+            return False, u"%s" % e
+        return True, ""
 
 
 class AbstractVote(models.Model):
     """
     Records user ratings as yes/no vote.
+
     * Only signed-in users can vote.
     * Each user can vote only once.
     """
     review = models.ForeignKey('reviews.ProductReview', related_name='votes')
-    user = models.ForeignKey('auth.User', related_name='review_votes')
+    user = models.ForeignKey(AUTH_USER_MODEL, related_name='review_votes')
     UP, DOWN = 1, -1
     VOTE_CHOICES = (
         (UP, _("Up")),
@@ -155,6 +196,18 @@ class AbstractVote(models.Model):
     def __unicode__(self):
         return u"%s vote for %s" % (self.delta, self.review)
 
+    def clean(self):
+        if not self.review.is_anonymous and self.review.user == self.user:
+            raise ValidationError(_(
+                "You cannot vote on your own reviews"))
+        if not self.user.id:
+            raise ValidationError(_(
+                "Only signed-in users can vote on reviews"))
+        previous_votes = self.review.votes.filter(user=self.user)
+        if len(previous_votes) > 0:
+            raise ValidationError(_(
+                "You can only vote once on a review"))
+
     def save(self, *args, **kwargs):
         super(AbstractVote, self).save(*args, **kwargs)
         self.review.update_totals()
