@@ -9,6 +9,7 @@ from PIL import Image
 from django.core.files import File
 from django.core.exceptions import FieldError
 from django.db.models import get_model
+from django.db.transaction import commit_on_success
 from django.utils.translation import ugettext_lazy as _
 
 from oscar.apps.catalogue.exceptions import (
@@ -27,6 +28,7 @@ class Importer(object):
         self.logger = logger
         self._field = field
 
+    @commit_on_success  # noqa (too complex (10))
     def handle(self, dirname):
         stats = {
             'num_processed': 0,
@@ -36,21 +38,29 @@ class Importer(object):
         if image_dir:
             for filename in filenames:
                 try:
-                    lookup_value = self._get_lookup_value_from_filename(filename)
+                    lookup_value \
+                        = self._get_lookup_value_from_filename(filename)
                     self._process_image(image_dir, filename, lookup_value)
                     stats['num_processed'] += 1
                 except Product.MultipleObjectsReturned:
-                    self.logger.warning("Multiple products matching %s='%s', skipping" % (self._field, lookup_value))
+                    self.logger.warning("Multiple products matching %s='%s',"
+                                        " skipping"
+                                        % (self._field, lookup_value))
                     stats['num_skipped'] += 1
                 except Product.DoesNotExist:
-                    self.logger.warning("No item matching %s='%s'" % (self._field, lookup_value))
+                    self.logger.warning("No item matching %s='%s'"
+                                        % (self._field, lookup_value))
                     stats['num_skipped'] += 1
                 except IdenticalImageError:
-                    self.logger.warning(" - Identical image already exists for %s='%s', skipping" % (self._field, lookup_value))
+                    self.logger.warning(" - Identical image already exists for"
+                                        " %s='%s', skipping"
+                                        % (self._field, lookup_value))
                     stats['num_skipped'] += 1
                 except IOError, e:
-                    raise ImageImportError(_('%(filename)s is not a valid image (%(error)s)') % {
-                        'filename': filename, 'error': e})
+                    raise ImageImportError(_('%(filename)s is not a valid'
+                                             ' image (%(error)s)')
+                                           % {'filename': filename,
+                                              'error': e})
                     stats['num_invalid'] += 1
                 except FieldError, e:
                     raise ImageImportError(e)
@@ -58,8 +68,10 @@ class Importer(object):
             if image_dir != dirname:
                 shutil.rmtree(image_dir)
         else:
-            raise InvalidImageArchive(_('%s is not a valid image archive') % dirname)
-        self.logger.info("Finished image import: %(num_processed)d imported, %(num_skipped)d skipped" % stats)
+            raise InvalidImageArchive(_('%s is not a valid image archive')
+                                      % dirname)
+        self.logger.info("Finished image import: %(num_processed)d imported,"
+                         " %(num_skipped)d skipped" % stats)
 
     def _get_image_files(self, dirname):
         filenames = []
@@ -67,7 +79,8 @@ class Importer(object):
         if image_dir:
             for filename in os.listdir(image_dir):
                 ext = os.path.splitext(filename)[1]
-                if os.path.isfile(os.path.join(image_dir, filename)) and ext in self.allowed_extensions:
+                if os.path.isfile(os.path.join(image_dir, filename)) \
+                        and ext in self.allowed_extensions:
                     filenames.append(filename)
         return image_dir, filenames
 
@@ -112,7 +125,7 @@ class Importer(object):
         kwargs = {self._field: lookup_value}
         item = Product._default_manager.get(**kwargs)
 
-        new_data = open(file_path).read()
+        new_data = open(file_path, 'rb').read()
         next_index = 0
         for existing in item.images.all():
             next_index = existing.display_order + 1
@@ -123,7 +136,7 @@ class Importer(object):
                 # File probably doesn't exist
                 existing.delete()
 
-        new_file = File(open(file_path))
+        new_file = File(open(file_path, 'rb'))
         im = ProductImage(product=item, display_order=next_index)
         im.original.save(filename, new_file, save=False)
         im.save()
