@@ -13,8 +13,10 @@ from django.utils.translation import ugettext_lazy as _
 from oscar.apps.customer.managers import CommunicationTypeManager
 from oscar.core.compat import AUTH_USER_MODEL
 
+
+# Only define custom UserManager/UserModel when Django >= 1.5
 if hasattr(auth_models, 'BaseUserManager'):
-    # Only define custom UserModel when Django >= 1.5
+
     class UserManager(auth_models.BaseUserManager):
 
         def create_user(self, email, password=None, **extra_fields):
@@ -49,7 +51,7 @@ if hasattr(auth_models, 'BaseUserManager'):
         An abstract base user suitable for use in Oscar projects.
 
         This is basically a copy of the core AbstractUser model but without a
-        username field
+        username field and with a unique index on the email field.
         """
         email = models.EmailField(_('email address'), unique=True)
         first_name = models.CharField(
@@ -83,6 +85,24 @@ if hasattr(auth_models, 'BaseUserManager'):
         def get_short_name(self):
             return self.first_name
 
+        def _migrate_alerts_to_user(self):
+            """
+            Transfer any active alerts linked to a user's email address to the
+            newly registered user.
+            """
+            ProductAlert = self.alerts.model
+            alerts = ProductAlert.objects.filter(
+                email=self.email, status=ProductAlert.ACTIVE)
+            alerts.update(user=self, key=None, email=None)
+
+        def save(self, *args, **kwargs):
+            super(AbstractUser, self).save(*args, **kwargs)
+            # Migrate any "anonymous" product alerts to the registered user
+            # Ideally, this would be done via a post-save signal. But we can't
+            # use get_user_model to wire up signals to custom user models
+            # see Oscar ticket #1127, Django ticket #19218
+            self._migrate_alerts_to_user()
+
 
 class AbstractEmail(models.Model):
     """
