@@ -1,10 +1,12 @@
 from django import forms
+from django.core.exceptions import ValidationError, MultipleObjectsReturned
 from django.forms.models import inlineformset_factory, BaseInlineFormSet
 from django.db.models import get_model
 from django.utils.translation import ugettext_lazy as _
-from treebeard.forms import MoveNodeForm
+from treebeard.forms import MoveNodeForm, movenodeform_factory
 
 from oscar.core.utils import slugify
+from oscar.core.loading import get_class
 from oscar.forms.widgets import ImageInput
 
 Product = get_model('catalogue', 'Product')
@@ -15,20 +17,23 @@ ProductAttributeValue = get_model('catalogue', 'ProductAttributeValue')
 ProductCategory = get_model('catalogue', 'ProductCategory')
 ProductImage = get_model('catalogue', 'ProductImage')
 ProductRecommendation = get_model('catalogue', 'ProductRecommendation')
+ProductSelect = get_class('dashboard.catalogue.widgets', 'ProductSelect')
+ProductSelectMultiple = get_class('dashboard.catalogue.widgets',
+                                  'ProductSelectMultiple')
 
 
-class CategoryForm(MoveNodeForm):
+class BaseCategoryForm(MoveNodeForm):
 
     def clean(self):
-        cleaned_data = super(CategoryForm, self).clean()
+        cleaned_data = super(BaseCategoryForm, self).clean()
 
         name = cleaned_data.get('name')
         ref_node_pk = cleaned_data.get('_ref_node_id')
         pos = cleaned_data.get('_position')
 
         if name and self.is_slug_conflicting(name, ref_node_pk, pos):
-            raise forms.ValidationError(_('Category with the given path'
-                                          ' already exists.'))
+            raise forms.ValidationError(
+                _('Category with the given path already exists.'))
         return cleaned_data
 
     def is_slug_conflicting(self, name, ref_node_pk, position):
@@ -43,7 +48,9 @@ class CategoryForm(MoveNodeForm):
             parent = None
 
         # build full slug
-        slug_prefix = (parent.slug + Category._slug_separator) if parent else ''
+        slug_prefix = ''
+        if parent:
+            slug_prefix = (parent.slug + Category._slug_separator)
         slug = '%s%s' % (slug_prefix, slugify(name))
 
         # check if slug is conflicting
@@ -56,8 +63,7 @@ class CategoryForm(MoveNodeForm):
                 return True
         return False
 
-    class Meta(MoveNodeForm.Meta):
-        model = Category
+CategoryForm = movenodeform_factory(Category, form=BaseCategoryForm)
 
 
 class ProductSearchForm(forms.Form):
@@ -67,18 +73,80 @@ class ProductSearchForm(forms.Form):
 
 class StockRecordForm(forms.ModelForm):
 
-    def __init__(self, product_class, *args, **kwargs):
-        self.product_class = product_class
+    def __init__(self, product_class, user, *args, **kwargs):
+        # The user kwarg is not used by stock StockRecordForm. We pass it
+        # anyway in case one wishes to customise the partner queryset
+        self.user = user
         super(StockRecordForm, self).__init__(*args, **kwargs)
 
         # If not tracking stock, we hide the fields
-        if not self.product_class.track_stock:
+        if not product_class.track_stock:
             del self.fields['num_in_stock']
             del self.fields['low_stock_threshold']
+        else:
+            self.fields['price_excl_tax'].required = True
+            self.fields['num_in_stock'].required = True
 
     class Meta:
         model = StockRecord
-        exclude = ('product', 'num_allocated', 'price_currency')
+        exclude = ('product', 'num_allocated')
+
+
+BaseStockRecordFormSet = inlineformset_factory(
+    Product, StockRecord, form=StockRecordForm, extra=1)
+
+
+class StockRecordFormSet(BaseStockRecordFormSet):
+
+    def __init__(self, product_class, user, *args, **kwargs):
+        self.user = user
+        self.require_user_stockrecord = not user.is_staff
+        self.product_class = product_class
+        super(StockRecordFormSet, self).__init__(*args, **kwargs)
+        self.set_initial_data()
+
+    def set_initial_data(self):
+        """
+        If user has only one partner associated, set the first
+        stock record's partner to it. Can't pre-select for staff users as
+        they're allowed to save a product without a stock record.
+
+        This is intentionally done after calling __init__ as passing initial
+        data to __init__ creates a form for each list item. So depending on
+        whether we can pre-select the partner or not, we'd end up with 1 or 2
+        forms for an unbound form.
+        """
+        if self.require_user_stockrecord:
+            try:
+                user_partner = self.user.partners.get()
+            except (Partner.DoesNotExist, MultipleObjectsReturned):
+                pass
+            else:
+                partner_field = self.forms[0].fields.get('partner', None)
+                if partner_field and partner_field.initial is None:
+                    partner_field.initial = user_partner
+
+    def _construct_form(self, i, **kwargs):
+        kwargs['product_class'] = self.product_class
+        kwargs['user'] = self.user
+        return super(StockRecordFormSet, self)._construct_form(
+            i, **kwargs)
+
+    def clean(self):
+        """
+        If the user isn't a staff user, this validation ensures that at least
+        one stock record's partner is associated with a users partners.
+        """
+        if any(self.errors):
+            return
+        if self.require_user_stockrecord:
+            stockrecord_partners = set([form.cleaned_data.get('partner', None)
+                                        for form in self.forms])
+            user_partners = set(self.user.partners.all())
+            if not user_partners & stockrecord_partners:
+                raise ValidationError(_("At least one stock record must be set"
+                                        " to a partner that you're associated"
+                                        " with."))
 
 
 def _attr_text_field(attribute):
@@ -88,7 +156,7 @@ def _attr_text_field(attribute):
 
 def _attr_textarea_field(attribute):
     return forms.CharField(label=attribute.name,
-               widget=forms.Textarea(),
+                           widget=forms.Textarea(),
                            required=attribute.required)
 
 
@@ -139,8 +207,28 @@ def _attr_numeric_field(attribute):
                             required=attribute.required)
 
 
+def _attr_file_field(attribute):
+    return forms.FileField(
+        label=attribute.name, required=attribute.required)
+
+
+def _attr_image_field(attribute):
+    return forms.ImageField(
+        label=attribute.name, required=attribute.required)
+
+
 class ProductForm(forms.ModelForm):
 
+    # We need a special field to distinguish between group and standalone
+    # products.  It's impossible to tell when the product is first created.
+    # This is quite clunky but will be replaced when #693 is complete.
+    is_group = forms.BooleanField(
+        label=_("Is group product?"),
+        required=False,
+        help_text=_(
+            "Check this if this product is a group/parent product "
+            "that has variants (eg different sizes/colours available)"))
+
     FIELD_FACTORIES = {
         "text": _attr_text_field,
         "richtext": _attr_textarea_field,
@@ -152,23 +240,50 @@ class ProductForm(forms.ModelForm):
         "multi_option": _attr_multi_option_field,
         "entity": _attr_entity_field,
         "numeric": _attr_numeric_field,
+        "file": _attr_file_field,
+        "image": _attr_image_field,
     }
 
-    def __init__(self, product_class, *args, **kwargs):
+    class Meta:
+        model = Product
+        exclude = ('slug', 'status', 'score', 'product_class',
+                   'recommended_products', 'product_options',
+                   'attributes', 'categories')
+        widgets = {
+            'parent': ProductSelect,
+            'related_products': ProductSelectMultiple,
+        }
+
+    def __init__(self, product_class, data=None, *args, **kwargs):
         self.product_class = product_class
         self.set_initial_attribute_values(kwargs)
-        super(ProductForm, self).__init__(*args, **kwargs)
-        self.add_attribute_fields()
+        super(ProductForm, self).__init__(data, *args, **kwargs)
+
+        # Set the initial value of the is_group field.  This isn't watertight:
+        # if the product is intended to be a parent product but doesn't have
+        # any variants then we can't distinguish it from a standalone product
+        # and this checkbox won't have the right value.  This will be addressed
+        # in #693
+        instance = kwargs.get('instance', None)
+        if instance:
+            self.fields['is_group'].initial = instance.is_group
+
+        # This is quite nasty.  We use the raw posted data to determine if the
+        # product is a group product, as this changes the validation rules we
+        # want to apply.
+        is_parent = data and data.get('is_group', '') == 'on'
+        self.add_attribute_fields(is_parent)
+
         related_products = self.fields.get('related_products', None)
-        if self.instance.pk is not None:
-            # prevent selecting itself as parent
-            parent = self.fields['parent']
-            parent.queryset = parent.queryset.exclude(pk=self.instance.pk)
+        parent = self.fields.get('parent', None)
+
+        if parent is not None:
+            parent.queryset = self.get_parent_products_queryset()
         if related_products is not None:
             related_products.queryset = self.get_related_products_queryset()
         if 'title' in self.fields:
             self.fields['title'].widget = forms.TextInput(
-                attrs={'autocompete': 'off'})
+                attrs={'autocomplete': 'off'})
 
     def set_initial_attribute_values(self, kwargs):
         if kwargs.get('instance', None) is None:
@@ -184,10 +299,13 @@ class ProductForm(forms.ModelForm):
             else:
                 kwargs['initial']['attr_%s' % attribute.code] = value
 
-    def add_attribute_fields(self):
+    def add_attribute_fields(self, is_parent=False):
         for attribute in self.product_class.attributes.all():
-            self.fields['attr_%s' % attribute.code] = \
-                    self.get_attribute_field(attribute)
+            self.fields['attr_%s' % attribute.code] \
+                = self.get_attribute_field(attribute)
+            # Attributes are not required for a parent product
+            if is_parent:
+                self.fields['attr_%s' % attribute.code].required = False
 
     def get_attribute_field(self, attribute):
         return self.FIELD_FACTORIES[attribute.type](attribute)
@@ -195,29 +313,34 @@ class ProductForm(forms.ModelForm):
     def get_related_products_queryset(self):
         return Product.browsable.order_by('title')
 
-    class Meta:
-        model = Product
-        exclude = ('slug', 'status', 'score', 'product_class',
-                   'recommended_products', 'product_options',
-                   'attributes', 'categories')
+    def get_parent_products_queryset(self):
+        """
+        :return: Canonical products excluding this product
+        """
+        # Not using Product.browsable because a deployment might override
+        # that manager to respect a status field or such like
+        queryset = Product._default_manager.filter(parent=None)
+        if self.instance.pk is not None:
+            # Prevent selecting itself as parent
+            queryset = queryset.exclude(pk=self.instance.pk)
+        return queryset
 
     def save(self):
-        object = super(ProductForm, self).save(False)
+        object = super(ProductForm, self).save(commit=False)
         object.product_class = self.product_class
         for attribute in self.product_class.attributes.all():
             value = self.cleaned_data['attr_%s' % attribute.code]
             setattr(object.attr, attribute.code, value)
         if not object.upc:
             object.upc = None
-        object.save()
-        if hasattr(self, 'save_m2m'):
-            self.save_m2m()
-        return object
 
-    def save_attributes(self, object):
-        for attribute in self.product_class.attributes.all():
-            value = self.cleaned_data['attr_%s' % attribute.code]
-            attribute.save_value(object, value)
+        if self.cleaned_data['is_group']:
+            # Don't validate attributes for parent products
+            object.save(validate_attributes=False)
+        else:
+            object.save()
+        self.save_m2m()
+        return object
 
     def clean(self):
         data = self.cleaned_data
@@ -257,7 +380,7 @@ class BaseProductCategoryFormSet(BaseInlineFormSet):
             form = self.forms[i]
             if (hasattr(form, 'cleaned_data')
                     and form.cleaned_data.get('category', None)
-                    and form.cleaned_data.get('DELETE', False) != True):
+                    and not form.cleaned_data.get('DELETE', False)):
                 num_categories += 1
         return num_categories
 
@@ -296,5 +419,14 @@ ProductImageFormSet = inlineformset_factory(
     Product, ProductImage, form=ProductImageForm, extra=2)
 
 
+class ProductRecommendationForm(forms.ModelForm):
+    class Meta:
+        model = ProductRecommendation
+        widgets = {
+            'recommendation': ProductSelect,
+        }
+
+
 ProductRecommendationFormSet = inlineformset_factory(
-    Product, ProductRecommendation, extra=5, fk_name="primary")
+    Product, ProductRecommendation, form=ProductRecommendationForm,
+    extra=5, fk_name="primary")
