@@ -1,8 +1,9 @@
 from django.views.generic import ListView
 from django.db.models import get_model
 from django import http
+from django.shortcuts import get_object_or_404
 
-from oscar.apps.offer.models import ConditionalOffer
+from oscar.apps.offer.models import ConditionalOffer, Range
 
 Product = get_model('catalogue', 'Product')
 
@@ -39,3 +40,23 @@ class OfferDetailView(ListView):
 
     def get_queryset(self):
         return self.offer.products()
+
+
+class RangeDetailView(ListView):
+    template_name = 'offer/range.html'
+    context_object_name = 'products'
+
+    def dispatch(self, request, *args, **kwargs):
+        self.range = get_object_or_404(
+            Range, slug=kwargs['slug'], is_public=True)
+        return super(RangeDetailView, self).dispatch(
+            request, *args, **kwargs)
+
+    def get_queryset(self):
+        products = self.range.included_products.all()
+        return products.order_by('rangeproduct__display_order')
+
+    def get_context_data(self, **kwargs):
+        ctx = super(RangeDetailView, self).get_context_data(**kwargs)
+        ctx['range'] = self.range
+        return ctx
