@@ -7,46 +7,54 @@ from django.utils.translation import ugettext_lazy as _
 
 
 class ExtendedURLValidator(validators.URLValidator):
+
+    def __init__(self, *args, **kwargs):
+        # 'verify_exists' has been removed in Django 1.5 and so we no longer
+        # pass it up to the core validator class
+        self.is_local_url = False
+        verify_exists = kwargs.pop('verify_exists', False)
+        super(ExtendedURLValidator, self).__init__(*args, **kwargs)
+        self.verify_exists = verify_exists
+
     def __call__(self, value):
         try:
             super(ExtendedURLValidator, self).__call__(value)
         except ValidationError:
+            # The parent validator will raise an exception if the URL does not
+            # exist and so we test here to see if the value is a local URL.
             if value:
                 self.validate_local_url(value)
             else:
                 raise
 
     def validate_local_url(self, value):
-        """
-        Validate local URL name
-        """
+        value = self.clean_url(value)
         try:
-            value = self.fix_local_url(value)
-            if self.verify_exists:
-                resolve(value)
-            self.is_local_url = True
+            resolve(value)
         except Http404:
             # We load flatpages here as it causes a circular reference problem
             # sometimes.  FlatPages is None if not installed
             FlatPage = get_model('flatpages', 'FlatPage')
             if FlatPage is not None:
-                for page in FlatPage.objects.all().only(('url')):
-                    if value == page.url:
-                        return
-            raise ValidationError(_('Specified page does not exist'))
+                try:
+                    FlatPage.objects.get(url=value)
+                except FlatPage.DoesNotExist:
+                    self.is_local_url = True
+                else:
+                    return
+            raise ValidationError(_('The URL "%s" does not exist') % value)
+        else:
+            self.is_local_url = True
 
-    def fix_local_url(self, value):
+    def clean_url(self, value):
         """
-        Puts preceding and trailing slashes to local URL name
+        Ensure url has a preceding slash and no query string
         """
         if value != '/':
             value = '/' + value.lstrip('/')
-
         q_index = value.find('?')
-
         if q_index > 0:
             value = value[:q_index]
-
         return value
 
 
@@ -54,8 +62,9 @@ class URLDoesNotExistValidator(ExtendedURLValidator):
 
     def __call__(self, value):
         """
-        Validate that the URL in *value* does not already exist. The
-        URL will be verified first and raises ``ValidationError`` when
+        Validate that the URLdoes not already exist.
+
+        The URL will be verified first and raises ``ValidationError`` when
         it is invalid. A valid URL is checked for existance and raises
         ``ValidationError`` if the URL already exists. Setting attribute
         ``verify_exists`` has no impact on validation.
@@ -64,25 +73,18 @@ class URLDoesNotExistValidator(ExtendedURLValidator):
 
         Returns ``None`` if URL is valid and does not exist.
         """
-        self.verify_exists = False
-
-        # calling ExtendedURLValidator twice instead of replicating its code
-        # and invert the cases when a ValidationError is returned seemes to
-        # be a much cleaner solution. Although this might not be the most
-        # efficient way of handling this, it should have not much of an impact
-        # due to its current application in flatpage creation.
         try:
-            super(URLDoesNotExistValidator, self).__call__(value)
-        except ValidationError:
-            raise ValidationError(_('Specified page does already exist'),
-                                  code='invalid')
-
-        # check if URL exists since it seems to be valid
-        try:
-            self.verify_exists = True
-            super(URLDoesNotExistValidator, self).__call__(value)
+            self.validate_local_url(value)
         except ValidationError:
+            # Page exists - that is what we want
             return
+        raise ValidationError(
+            _('Specified page already exists!'), code='invalid')
 
-        raise ValidationError(_('Specified page does already exist'),
-                              code='invalid')
+
+def non_whitespace(value):
+    stripped = value.strip()
+    if not stripped:
+        raise ValidationError(
+            _("This field is required"))
+    return stripped
